/***********************************************************************;
 * Copyright (c) 2018, Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 ***********************************************************************/

#include "sapi/tpm20.h"
#include "sysapi_util.h"

void Unmarshal_TPML_AC_CAPABILITIES(
    TSS2_SYS_CONTEXT *sysContext,
    TPML_AC_CAPABILITIES *capabilityData)
{
    UINT32 i;

    if (SYS_CONTEXT->rval != TSS2_RC_SUCCESS)
        return;

    if (!capabilityData)
        return;

    Unmarshal_UINT32(SYS_CONTEXT->tpmOutBuffPtr, SYS_CONTEXT->maxResponseSize,
                     &SYS_CONTEXT->nextData, &capabilityData->count, &SYS_CONTEXT->rval);

    for (i = 0; i < capabilityData->count && SYS_CONTEXT->rval == TSS2_RC_SUCCESS; i++)
    {
        Unmarshal_TPMS_AC_OUTPUT(sysContext, &capabilityData->acCapabilities[i]);
    }

    return;
}
